package jsr308.javac;

import java.io.*;
import java.net.*;
import java.util.*;

import jsr308.*;
import jsr308.util.*;

import org.eclipse.core.runtime.*;
import org.osgi.framework.*;

/**
 * Runs the compiler and parses the output.
 */
public class CommandlineJavacRunner{
    private static final String JAVAC_LOCATION = "lib/javac.jar";
    public static boolean VERBOSE = false;

    public List<JavacError> callJavac(List<String> fileNames, String processor, String classpath){
        try{
            String[] cmd = options(fileNames, processor, classpath);
            if (VERBOSE)
                System.out.println(Arrays.toString(cmd));
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(baos);

            if (VERBOSE)
                System.out.println(toStringNoCommas(cmd));
            Command.exec(cmd, ps);
            return JavacError.parse(baos.toString());
        }catch (IOException e){
            Activator.getDefault().logException(e, "Error calling javac");
            return null;
        }
    }

    // None of the Arrays.toString decorations like [, ]
    private String toStringNoCommas(String[] strings){
        StringBuilder b = new StringBuilder();
        for (String string : strings){
            b.append(string).append(" ");
        }
        return b.toString();
    }

    private String[] options(List<String> fileNames, String processor, String classpath) throws IOException{
        List<String> opts = new ArrayList<String>();
        opts.addAll(javaCommand());
        if (VERBOSE)
            opts.add("-verbose");
        opts.add("-proc:only");
        opts.add("-classpath");
        opts.add(classpath(classpath));
        opts.add("-typeprocessor");
        opts.add(processor);
        opts.addAll(fileNames);
        return opts.toArray(new String[opts.size()]);
    }

    private List<String> javaCommand() throws IOException{
        if (System.getProperty("jsr308_javac") != null)
            return Collections.singletonList(System.getProperty("jsr308_javac_cmd"));
        if (System.getenv("jsr308_javac") != null)
            return Collections.singletonList(System.getenv("jsr308_javac_cmd"));

        List<String> opts = new ArrayList<String>();
        System.out.println(System.getProperties());
        System.out.println(System.getenv());
        opts.add(javaVM());
        opts.add("-ea:com.sun.tools");
        opts.add("-Xbootclasspath/p:" + javacJARlocation());
        opts.add("-jar");
        opts.add(javacJARlocation());
        return opts;
    }

    private String javaVM(){
        String sep = System.getProperty("file.separator");
        return System.getProperty("java.home") + sep + "bin" + sep + "java";
    }

    private String classpath(String classpath){
        if (System.getProperty("jsr308_checkers") != null)
            // prepend to the end
            return classpath + ":" + System.getProperty("jsr308_checkers");
        if (System.getenv("jsr308_checkers") != null)
            // prepend to the end
            return classpath + ":" + System.getenv("jsr308_checkers");
        return classpath;
    }

    private String javacJARlocation() throws IOException{
        Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);

        Path javacJAR = new Path(JAVAC_LOCATION);
        URL javacJarURL = FileLocator.toFileURL(FileLocator.find(bundle, javacJAR, null));
        return javacJarURL.getPath();
    }

    // This used to be used. Now we just scan the classpath. The checkers.jar must be on the classpath anyway.
    // XXX The problem is what to do if the checkers.jar on the classpath is different from the one in the plugin.
    // private String checkersJARlocation() throws IOException{
    // Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
    //
    // Path checkersJAR = new Path(CHECKERS_LOCATION);
    // URL checkersJarURL = FileLocator.toFileURL(FileLocator.find(bundle, checkersJAR, null));
    //
    // return checkersJarURL.getPath();
    // }
}