package jsr308.javac;

import java.io.*;
import java.util.*;

import jsr308.util.*;

/**
 * Error reported by javac. Created by parsing javac output.
 */
public class JavacError{
    private static final boolean VERBOSE = false;
    private final File file;
    private final int lineNumber;
    private final String message;

    public JavacError(File file, int lineNumber, String message){
        this.file = file;
        this.lineNumber = lineNumber;
        this.message = message;
    }

    /**
     * Parses javac output and converts to a list of errors.
     */
    // XXX very nasty code - needs cleanup
    public static List<JavacError> parse(String javacoutput){
        if (VERBOSE)
            System.out.println("javac output:\n" + javacoutput);
        if (javacoutput == null)
            return null;
        List<JavacError> result = new ArrayList<JavacError>();
        List<String> lines = Arrays.asList(javacoutput.split(Util.NL));
        Iterator<String> iter = lines.iterator();
        if (!iter.hasNext())
            return result;
        String line = iter.next();
        do{
            String[] segments = line.split(":");
            if (segments.length != 3){
                if (iter.hasNext())
                    line = iter.next();
                continue; // ??
            }
            try{
                int lineNumber = Integer.parseInt(segments[1]);
                StringBuilder msg = new StringBuilder();
                msg.append(segments[2].trim());
                boolean foundNextEntry = false;
                while (!foundNextEntry && iter.hasNext()){
                    line = iter.next();
                    foundNextEntry = line.split(":").length == 3 && new File(line.split(":")[0]).exists();
                    if (!foundNextEntry && !line.matches("^[0-9]+ error*$")){ // don't include the last line
                        msg.append(Util.NL).append(line);
                    }
                }
                File f = new File(segments[0]);
                result.add(new JavacError(f, lineNumber, msg.toString()));
            }catch (NumberFormatException e){
                if (iter.hasNext())
                    line = iter.next();
                continue;
            }
        }while (iter.hasNext());
        return result;
    }

    public File getFile(){
        return file;
    }

    public int getLineNumber(){
        return lineNumber;
    }

    public String getMessage(){
        return message;
    }

    @Override
    public String toString(){
        return file.getPath() + ":" + lineNumber + ": " + message;
    }
}
